@@ -16,60 +16,54 @@ pub struct Domain {
 
 	pub url:    Option<Url>,
 	pub domain: String,
-	pub head:   String,
-	pub tail:   String,
+	addr_key:   [u8; 4],
 }
 
 impl Domain {
-	fn _new(domain: String, shards_total: usize, url: Option<Url>, is_discovery: bool) -> Domain {
-		let split: Vec<String> = domain.split('.').map(String::from).collect();
+	pub fn new(
+		domain: String,
+		addrs: Vec<[u8; 4]>,
+		shards_total: usize,
+		addr_key_mask: u8,
+		url: Option<Url>,
+		is_discovery: bool,
+	) -> Domain {
+		let mut addrs = addrs;
+		addrs.sort_unstable();
+		let mut addr = addrs.into_iter().next().unwrap_or([255, 255, 255, 255]);
 
-		if split.len() < 2 {
-			return Domain {
-				is_discovery,
-				shard: 0,
-				shards_total,
-				url,
-				domain: domain.clone(),
-				head: domain,
-				tail: "".into(),
+		let mut left = addr_key_mask;
+		for a in &mut addr {
+			if left >= 8 {
+				left -= 8;
+			} else {
+				let mut mask = 0;
+				for k in 0..left {
+					mask |= 1 << k;
+				}
+				*a &= mask;
 			}
 		}
-		if split.len() == 2 {
-			return Domain { is_discovery, shard: 0, shards_total, url, domain, head: split.join("."), tail: "".into() }
-		}
 
-		Domain {
-			is_discovery,
-			shard: 0,
-			shards_total,
-			url,
-			domain,
-			head: split[split.len() - 2..].join("."),
-			tail: split[..split.len() - 2].join("."),
-		}
-	}
-
-	pub fn new(domain: String, shards_total: usize, url: Option<Url>, is_discovery: bool) -> Domain {
-		let mut domain = Domain::_new(domain, shards_total, url, is_discovery);
+		let mut domain = Domain { addr_key: addr, is_discovery, shard: 0, shards_total, url, domain };
 		domain.calc_shard();
 		domain
 	}
 
 	fn calc_shard(&mut self) {
 		let mut hasher = crc32fast::Hasher::new();
-		hasher.update(self.head.as_bytes());
+		hasher.update(&self.addr_key);
 		self.shard = (hasher.finalize() % self.shards_total as u32 + 1) as u16;
 	}
 }
 
 #[derive(Clone, Debug, Serialize, Deserialize, Row)]
 pub struct DomainDBEntry {
-	pub shard:       u16,
-	pub domain:      String,
-	pub domain_tail: String,
-	pub created_at:  u32,
-	pub updated_at:  u32,
+	pub shard:      u16,
+	pub addr_key:   [u8; 4],
+	pub domain:     String,
+	pub created_at: u32,
+	pub updated_at: u32,
 }
 
 impl From<Domain> for DomainDBEntry {
@@ -77,11 +71,11 @@ impl From<Domain> for DomainDBEntry {
 		let now = now();
 
 		Self {
-			shard:       s.shard,
-			domain:      s.head,
-			domain_tail: s.tail,
-			created_at:  now,
-			updated_at:  if s.is_discovery { 644616000 } else { now },
+			shard:      s.shard,
+			addr_key:   s.addr_key,
+			domain:     s.domain,
+			created_at: now,
+			updated_at: if s.is_discovery { 644616000 } else { now },
 		}
 	}
 }
